
type identifier = Names.identifier

type ('term, 'expr, 'value) value_node =
        | CLVvar of int
        | CLVnat of 'term
        | CLVint of 'term
        | CLVbool of 'term
        | CLVpack of 'term * 'value * 'term
        | CLVtuple of 'value list
        | CLVfix of identifier * (identifier * 'term) list *
                    identifier * 'term * 'expr
        | CLVapp of 'value * 'term
and ('term, 'expr, 'value) expr_node =
        | CLEapp of 'value * 'value
        | CLEval of identifier * 'value * 'expr
        | CLEunpack of identifier * identifier * 'value * 'expr
        | CLEsel of identifier * 'term * 'value * 'value * 'expr
        | CLEunop of identifier * Clops.unop * 'value * 'expr
        | CLEbinop of identifier * Clops.binop * 'value * 'value * 'expr
        | CLEif of 'term * 'term *
                   'value * identifier * 'expr * identifier * 'expr
and value = (Prelude.term, expr, value) value_node
and expr = (Prelude.term, expr, value) expr_node

val clvvar : int -> value
val clvnat : Prelude.term -> value
val clvint : Prelude.term -> value
val clvbool : Prelude.term -> value
val clvpack : Prelude.term -> value -> Prelude.term -> value
val clvtuple : value list -> value
val clvfix : identifier -> (identifier * Prelude.term) list ->
             identifier -> Prelude.term -> expr -> value
val clvapp : value -> Prelude.term -> value

val cleapp : value -> value -> expr
val cleval : identifier -> value -> expr -> expr
val cleunpack : identifier -> identifier -> value -> expr -> expr
val clesel : identifier -> Prelude.term -> value -> value -> expr -> expr
val cleunop : identifier -> Clops.unop -> value -> expr -> expr
val clebinop : identifier -> Clops.binop -> value -> value -> expr -> expr
val cleif : Prelude.term -> Prelude.term ->
            value -> identifier -> expr -> identifier -> expr -> expr

val cpsval_to_closval : Cpsexpr.value -> value

val type_of_expr : Environ.env -> Environ.env -> expr -> unit
val type_of_value : Environ.env -> Environ.env -> value -> Prelude.term

val pr_value : Environ.env -> Environ.env -> value -> Pp.std_ppcmds

