
type identifier = Names.identifier

type ('term, 'expr, 'value) value_node =
        | CVvar of int
        | CVnat of 'term
        | CVint of 'term
        | CVbool of 'term
        | CVpack of 'term * 'value * 'term
        | CVtuple of 'value list
        | CVfix of identifier * (identifier * 'term) list *
                   identifier * 'term * 'expr *
                   (identifier * 'term) list * (identifier * 'term) list
and ('term, 'expr, 'value) expr_node =
        | CEapp of 'value * 'term list * 'value
        | CEval of identifier * 'value * 'expr
        | CEunpack of identifier * identifier * 'value * 'expr
        | CEsel of identifier * 'term * 'value * 'value * 'expr
        | CEunop of identifier * Clops.unop * 'value * 'expr
        | CEbinop of identifier * Clops.binop * 'value * 'value * 'expr
        | CEif of 'term * 'term *
                  'value * identifier * 'expr * identifier * 'expr
and value = (Prelude.term, expr, value) value_node
and expr = (Prelude.term, expr, value) expr_node

val cvvar : int -> value
val cvnat : Prelude.term -> value
val cvint : Prelude.term -> value
val cvbool : Prelude.term -> value
val cvpack : Prelude.term -> value -> Prelude.term -> value
val cvtuple : value list -> value
val cvfix : identifier -> (identifier * Prelude.term) list ->
            identifier -> Prelude.term -> expr ->
            (identifier * Prelude.term) list ->
            (identifier * Prelude.term) list ->
            value

val ceapp : value -> Prelude.term list -> value -> expr
val ceval : identifier -> value -> expr -> expr
val ceunpack : identifier -> identifier -> value -> expr -> expr
val cesel : identifier -> Prelude.term -> value -> value -> expr -> expr
val ceunop : identifier -> Clops.unop -> value -> expr -> expr
val cebinop : identifier -> Clops.binop -> value -> value -> expr -> expr
val ceif : Prelude.term -> Prelude.term ->
           value -> identifier -> expr -> identifier -> expr -> expr

val aexpr_to_cps : Expr.aexpr -> value

val type_of_expr : Environ.env -> Environ.env -> expr -> expr
val type_of_value : Environ.env -> Environ.env -> value ->
                    (value * Prelude.term)

val pr_value : Environ.env -> Environ.env -> value -> Pp.std_ppcmds

val pr_value_tex : Environ.env -> Environ.env -> value -> Pp.std_ppcmds

