Definition decidable (P : Prop) (b : bool) : Prop :=
  if b then P else ~P
.

Lemma Z_ge_refl :
  forall n : Z, (n >= n)%Z
.
Proof.
  auto with zarith.
Qed.

Lemma geDecidablePrf :
  forall n m : Z, decidable (Zge n m) (Zge_bool n m)
.
Proof.
  intros; unfold decidable, Zge, Zge_bool;
    case (Zcompare n m); [ discriminate | auto | discriminate ].
Qed.

Lemma gtDecidablePrf :
  forall n m : Z, decidable (Zgt n m) (Zgt_bool n m)
.
Proof.
  intros; unfold decidable, Zgt, Zgt_bool;
    case (Zcompare n m); [ discriminate | discriminate | auto ].
Qed.

Definition leRoot (r : Z) (x : Z) : Prop :=
  (r >= 0 /\ r*r <= x)%Z
.

Definition isRoot (r : Z) (x : Z) : Prop :=
  leRoot r x /\ ((r+1)*(r+1) > x)%Z
.

Lemma Zplus_ge_compat :
  forall n m p q : Z, (n >= m -> p >= q -> n + p >= m + q)%Z.
Proof.
  intros n m p q; intros H1 H2; apply Zle_ge; apply Zplus_le_compat;
    apply Zge_le; assumption.
Qed.

%%%

poly nT : Z.
poly nP : (nT >= 0)%Z.
lambda n : sint nT. 
  (fix loop : forallT Z (fun yT => 
                forallT (leRoot yT nT) (fun yP =>
                  arrow 
                    (sint yT)
                    (existsT Z (fun xT : Z =>
                      (existsT (isRoot xT nT) (fun xP => sint xT)))))).
    poly yT : Z.
    poly yP : leRoot yT nT.
    lambda y : sint yT.
      if [decidable ((yT+1)*(yT+1) > nT)%Z,
          gtDecidablePrf ((yT+1)*(yT+1))%Z nT]
         ((y + cint[1]) * (y + cint[1]) > n,
           p1P. pack(yT, pack(conj yP p1P, y)
                         as existsT (isRoot yT nT) (fun xP => sint yT))
                as existsT Z (fun xT =>
                     existsT (isRoot xT nT) (fun xP => sint xT)),
           p2P. loop {(yT+1)%Z}
                     {conj (Zplus_ge_compat yT 0%Z 1%Z 0%Z
                             (proj1 yP) (geDecidablePrf 1%Z 0%Z))
                           (Znot_gt_le ((yT+1)*(yT+1))%Z nT p2P)}
                     (y + cint[1])))
    {0%Z} {conj (Z_ge_refl 0%Z) (Zge_le nT 0%Z nP)} cint[0]
