
open Prelude
open Clops

type identifier = Names.identifier

type ('term, 'expr) expr_node =
          Evar of int
        | Ecnat of 'term
        | Ecint of 'term
        | Ecbool of 'term
        | Elambda of identifier * 'term * 'expr
        | Epoly of identifier * 'term * 'expr
        | Efix of identifier * 'term * 'expr
        | Eapp of 'expr * 'expr
        | Etapp of 'expr * 'term
        | Eif of 'term * 'term * 'expr * identifier * 'expr * identifier * 'expr
        | Etuple of 'expr list
        | Esel of 'term * 'expr * 'expr
        | Epack of 'term * 'expr * 'term
        | Eunpack of 'expr * identifier * identifier * 'expr
        | Eunop of unop * 'expr
        | Ebinop of binop * 'expr * 'expr

type aexpr = (term, aexpr) expr_node * term

val aevar : int -> term -> aexpr
val aecnat : term -> term -> aexpr
val aecint : term -> term -> aexpr
val aecbool : term -> term -> aexpr
val aelambda : identifier -> term -> aexpr -> term -> aexpr
val aepoly : identifier -> term -> aexpr -> term -> aexpr
val aefix : identifier -> term -> aexpr -> term -> aexpr
val aeapp : aexpr -> aexpr -> term -> aexpr
val aetapp : aexpr -> term -> term -> aexpr
val aeif : term -> term ->
      aexpr -> identifier -> aexpr -> identifier -> aexpr -> term -> aexpr
val aetupple : aexpr list -> term -> aexpr
val aesel : term -> aexpr -> aexpr -> term -> aexpr
val aepack : term -> aexpr -> term -> term -> aexpr
val aeunpack : aexpr -> identifier -> identifier -> aexpr -> term -> aexpr
val aeunop : unop -> aexpr -> term -> aexpr
val aebinop : binop -> aexpr -> aexpr -> term -> aexpr

val pr_aexpr : Environ.env -> Environ.env -> aexpr -> Pp.std_ppcmds

